@@ -10,7 +10,7 @@ module ViewComponent # :nodoc:
 
     def render_in(view_context, &block)
       ActiveSupport::Notifications.instrument(
-        "!render.view_component",
+        notification_name,
         {
           name: self.class.name,
           identifier: self.class.identifier
@@ -19,5 +19,13 @@ module ViewComponent # :nodoc:
         super(view_context, &block)
       end
     end
+
+    private
+
+    def notification_name
+      return "!render.view_component" if ViewComponent::Base.config.use_deprecated_instrumentation_name
+
+      "render.view_component"
+    end
   end
 end
