@@ -2,11 +2,12 @@
 
 module ViewComponent
   module VERSION
-    MAJOR = 2
-    MINOR = 74
-    PATCH = 1
+    MAJOR = 3
+    MINOR = 8
+    PATCH = 0
+    PRE = nil
 
-    STRING = [MAJOR, MINOR, PATCH].join(".")
+    STRING = [MAJOR, MINOR, PATCH, PRE].compact.join(".")
   end
 end
 
